@@ -1,18 +1,22 @@
 blup.rma.uni <- function(x, level, digits, transf, targs, ...) {
 
-   if (!inherits(x, "rma.uni"))
-      stop("Argument 'x' must be an object of class \"rma.uni\".")
+   mstyle <- .get.mstyle("crayon" %in% .packages())
+
+   .chkclass(class(x), must="rma.uni", notav="rma.uni.selmodel")
 
    na.act <- getOption("na.action")
 
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
-      stop("Unknown 'na.action' specified under options().")
+      stop(mstyle$stop("Unknown 'na.action' specified under options()."))
 
    if (missing(level))
       level <- x$level
 
-   if (missing(digits))
-      digits <- x$digits
+   if (missing(digits)) {
+      digits <- .get.digits(xdigits=x$digits, dmiss=TRUE)
+   } else {
+      digits <- .get.digits(digits=digits, xdigits=x$digits, dmiss=FALSE)
+   }
 
    if (missing(transf))
       transf <- FALSE
@@ -20,10 +24,10 @@ blup.rma.uni <- function(x, level, digits, transf, targs, ...) {
    if (missing(targs))
       targs <- NULL
 
-   level <- ifelse(level > 1, (100-level)/100, ifelse(level > .5, 1-level, level))
+   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
 
    if (is.element(x$test, c("knha","adhoc","t"))) {
-      crit <- qt(level/2, df=x$dfs, lower.tail=FALSE)
+      crit <- qt(level/2, df=x$ddf, lower.tail=FALSE)
    } else {
       crit <- qnorm(level/2, lower.tail=FALSE)
    }
@@ -31,7 +35,7 @@ blup.rma.uni <- function(x, level, digits, transf, targs, ...) {
    ### TODO: check computations for user-defined weights
 
    if (!is.null(x$weights) || !x$weighted)
-      stop("Extraction of random effects for models with non-standard weights not currently implemented.")
+      stop(mstyle$stop("Extraction of random effects not available for models with non-standard weights."))
 
    #########################################################################
 
@@ -41,7 +45,7 @@ blup.rma.uni <- function(x, level, digits, transf, targs, ...) {
    ### see Appendix in: Raudenbush, S. W., & Bryk, A. S. (1985). Empirical
    ### Bayes meta-analysis. Journal of Educational Statistics, 10(2), 75-98
 
-   li <- x$tau2 / (x$tau2 + x$vi.f)
+   li <- x$tau2.f / (x$tau2.f + x$vi.f)
 
    for (i in seq_len(x$k.f)[x$not.na]) { ### note: skipping NA cases
       Xi <- matrix(x$X.f[i,], nrow=1)
@@ -91,7 +95,7 @@ blup.rma.uni <- function(x, level, digits, transf, targs, ...) {
    }
 
    if (na.act == "na.fail" && any(!x$not.na))
-      stop("Missing values in results.")
+      stop(mstyle$stop("Missing values in results."))
 
    #########################################################################
 
