@@ -1,29 +1,23 @@
 forest.rma <- function(x,
-annotate=TRUE, addfit=TRUE, addcred=FALSE, showweights=FALSE, header=FALSE,
-xlim, alim, clim, ylim, top=3, at, steps=5, level=x$level, refline=0, digits=2L, width,
+annotate=TRUE, addfit=TRUE, addpred=FALSE, showweights=FALSE, header=FALSE,
+xlim, alim, olim, ylim, top=3, at, steps=5, level=x$level, refline=0, digits=2L, width,
 xlab, slab, mlab, ilab, ilab.xpos, ilab.pos, order,
 transf, atransf, targs, rows,
-efac=1, pch=15, psize, col, border, lty, fonts,
-cex, cex.lab, cex.axis, annosym, ...) {
+efac=1, pch=15, psize, plim=c(0.5,1.5), colout, col, border,
+lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
 
    #########################################################################
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (!inherits(x, "rma"))
-      stop(mstyle$stop("Argument 'x' must be an object of class \"rma\"."))
-
-   if (inherits(x, "rma.ls"))
-      stop(mstyle$stop("Method not available for objects of class \"rma.ls\"."))
+   .chkclass(class(x), must="rma", notav="rma.ls")
 
    na.act <- getOption("na.action")
+   on.exit(options(na.action=na.act))
 
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
       stop(mstyle$stop("Unknown 'na.action' specified under options()."))
 
-   #if (!is.null(order))
-   #   order <- match.arg(order, c("obs", "fit", "prec", "resid", "rstandard", "abs.resid", "abs.rstandard"))
-
    if (missing(transf))
       transf <- FALSE
 
@@ -54,6 +48,9 @@ cex, cex.lab, cex.axis, annosym, ...) {
    if (missing(order))
       order <- NULL
 
+   if (missing(colout))
+      colout <- "black"
+
    if (missing(psize))
       psize <- NULL
 
@@ -66,32 +63,44 @@ cex, cex.lab, cex.axis, annosym, ...) {
    if (missing(cex.axis))
       cex.axis <- NULL
 
+   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+
+   ### digits[1] for annotations, digits[2] for x-axis labels
+   ### note: digits can also be a list (e.g., digits=list(2L,3)); trailing 0's are dropped for intergers
+
+   if (length(digits) == 1L)
+      digits <- c(digits,digits)
+
+   ############################################################################
+
    ### set default colors if user has not specified 'col' and 'border' arguments
 
    if (x$int.only) {
 
       if (missing(col)) {
-         col <- c("black", "gray50") ### 1st color for summary polygon, 2nd color for credibility interval
+         col <- c("black", "gray50") # 1st color for summary polygon, 2nd color for prediction interval line
       } else {
-         if (length(col) == 1L)      ### if user only specified one value, assume it is for summary polygon
+         if (length(col) == 1L)      # if user only specified one value, assume it is for the summary polygon
             col <- c(col, "gray50")
       }
 
       if (missing(border))
-         border <- "black"           ### border color of summary polygon
+         border <- "black"           # border color of summary polygon
 
    } else {
 
       if (missing(col))
-         col <- "gray"               ### color of fitted values
+         col <- "gray"               # color of fitted values
 
       if (missing(border))
-         border <- "gray"            ### border color of fitted values
+         border <- "gray"            # border color of fitted values
 
    }
 
+   ### set default line types if user has not specified 'lty' argument
+
    if (missing(lty)) {
-      lty <- c("solid", "dotted", "solid") ### 1st value = CIs, 2nd value = credibility interval, 3rd = horizontal line(s)
+      lty <- c("solid", "dotted", "solid") # 1st value = CIs, 2nd value = prediction interval, 3rd = horizontal line(s)
    } else {
       if (length(lty) == 1L)
          lty <- c(lty, "dotted", "solid")
@@ -105,16 +114,18 @@ cex, cex.lab, cex.axis, annosym, ...) {
       efac <- rep(efac, 3)
 
    if (length(efac) == 2L)
-      efac <- c(efac[1], efac[1], efac[2])
+      efac <- c(efac[1], efac[1], efac[2]) # if 2 values specified: 1st = CI end lines and arrows, 2nd = summary polygon or fitted polygons
 
    ### annotation symbols vector
 
    if (missing(annosym))
-      annosym <- c(" [", ", ", "]")
-   if (length(annosym) != 3L)
+      annosym <- c(" [", ", ", "]", "-") # 4th element for minus sign symbol
+   if (length(annosym) == 3L)
+      annosym <- c(annosym, "-")
+   if (length(annosym) != 4L)
       stop(mstyle$stop("Argument 'annosym' must be a vector of length 3."))
 
-   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+   ### get measure from object
 
    measure <- x$measure
 
@@ -145,32 +156,51 @@ cex, cex.lab, cex.axis, annosym, ...) {
       }
    }
 
+   if (!annotate)
+      header.right <- NULL
+
    ddd <- list(...)
 
-   lplot     <- function(..., textpos) plot(...)
-   labline   <- function(..., textpos) abline(...)
-   lsegments <- function(..., textpos) segments(...)
-   laxis     <- function(..., textpos) axis(...)
-   lmtext    <- function(..., textpos) mtext(...)
-   lpolygon  <- function(..., textpos) polygon(...)
-   ltext     <- function(..., textpos) text(...)
-   lpoints   <- function(..., textpos) points(...)
+   if (!is.null(ddd$addcred))
+      addpred <- ddd$addcred
+
+   if (is.null(ddd$pi.type)) {
+      pi.type <- "default"
+   } else {
+      pi.type <- ddd$pi.type
+   }
+
+   if (is.null(ddd$decreasing)) {
+      decreasing <- FALSE
+   } else {
+      decreasing <- ddd$decreasing
+   }
+
+   if (!is.null(ddd$clim))
+      olim <- ddd$clim
+
+   lplot     <- function(..., textpos, addcred, pi.type, decreasing, clim) plot(...)
+   labline   <- function(..., textpos, addcred, pi.type, decreasing, clim) abline(...)
+   lsegments <- function(..., textpos, addcred, pi.type, decreasing, clim) segments(...)
+   laxis     <- function(..., textpos, addcred, pi.type, decreasing, clim) axis(...)
+   lmtext    <- function(..., textpos, addcred, pi.type, decreasing, clim) mtext(...)
+   lpolygon  <- function(..., textpos, addcred, pi.type, decreasing, clim) polygon(...)
+   ltext     <- function(..., textpos, addcred, pi.type, decreasing, clim) text(...)
+   lpoints   <- function(..., textpos, addcred, pi.type, decreasing, clim) points(...)
 
    ### TODO: remove this when there is a weights() function for 'rma.glmm' objects
    if (inherits(x, "rma.glmm") && showweights)
       stop(mstyle$stop("Option 'showweights=TRUE' not possible for 'rma.glmm' objects."))
 
+   ### TODO: remove this when there is a weights() function for 'rma.uni.selmodel' objects
+   if (inherits(x, "rma.uni.selmodel") && showweights)
+      stop(mstyle$stop("Option 'showweights=TRUE' not possible for 'rma.uni.selmodel' objects."))
+
    if (!is.null(ddd$subset))
-      stop(mstyle$stop("Function does not have a 'subset' argument (could use 'order' argument instead)."))
+      stop(mstyle$stop("Function does not have a 'subset' argument."))
 
    #########################################################################
 
-   ### digits[1] for annotations, digits[2] for x-axis labels
-   ### note: digits can also be a list (e.g., digits=list(2L,3))
-
-   if (length(digits) == 1L)
-      digits <- c(digits,digits)
-
    ### extract data and study labels
    ### note: yi.f/vi.f and pred may contain NAs
 
@@ -178,125 +208,172 @@ cex, cex.lab, cex.axis, annosym, ...) {
    vi <- x$vi.f
    X  <- x$X.f
 
-   k <- length(yi)                              ### length of yi.f
+   k <- length(yi)                              # length of yi.f
+
+   ### note: slab (if specified), ilab (if specified), pch (if vector), psize (if
+   ###       vector), colout (if vector), order (if vector) must have the same
+   ###       length as the original dataset
 
    if (missing(slab)) {
+
       if (x$slab.null) {
-         slab <- paste("Study", x$slab)         ### x$slab is always of length yi.f (i.e., NAs also have an slab)
+         slab <- paste("Study", x$ids)          # x$ids is always of length yi.f (i.e., NAs also have an id)
       } else {
-         slab <- x$slab                         ### note: slab must have same length as yi.f in rma object
-      }                                         ### even when fewer studies used for model fitting (due to NAs)
+         slab <- x$slab                         # x$slab is always of length yi.f (i.e., NAs also have a study label)
+      }
+
    } else {
-      if (length(slab) == 1L && is.na(slab))
-         slab <- rep("", k)
+
+      if (is.null(slab) || (length(slab) == 1L && is.na(slab))) # slab=NULL or slab=NA can be used to suppress study labels
+         slab <- rep("", x$k.all)
+
+      if (length(slab) != x$k.all)
+         stop(mstyle$stop(paste0("Length of the 'slab' argument (", length(slab), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
+
+      if (!is.null(x$subset))
+         slab <- slab[x$subset]
+
    }
 
-   if (length(yi) != length(slab))
-      stop(mstyle$stop("Number of outcomes does not correspond to the length of the 'slab' argument."))
+   if (!is.null(ilab)) {
 
-   if (is.null(dim(ilab)))                      ### note: ilab must have same length as yi.f in rma object
-      ilab <- cbind(ilab)                       ### even when fewer studies used for model fitting
+      if (is.null(dim(ilab)))
+         ilab <- cbind(ilab)
 
-   if (length(pch) == 1L)                       ### note: pch must have same length as yi.f in rma object
-      pch <- rep(pch, k)                        ### or be equal to a single value (which is then repeated)
+      if (nrow(ilab) != x$k.all)
+         stop(mstyle$stop(paste0("Length of the 'ilab' argument (", nrow(ilab), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
 
-   if (length(pch) != length(yi))
-      stop(mstyle$stop("Number of outcomes does not correspond to the length of the 'pch' argument."))
+      if (!is.null(x$subset))
+         ilab <- ilab[x$subset,,drop=FALSE]
+
+   }
+
+   if (length(pch) == 1L)
+      pch <- rep(pch, x$k.all)
+
+   if (length(pch) != x$k.all)
+      stop(mstyle$stop(paste0("Length of the 'pch' argument (", length(pch), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
+
+   if (!is.null(x$subset))
+      pch <- pch[x$subset]
+
+   if (!is.null(psize)) {
+
+      if (length(psize) == 1L)
+         psize <- rep(psize, x$k.all)
+
+      if (length(psize) != x$k.all)
+         stop(mstyle$stop(paste0("Length of the 'psize' argument (", length(psize), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
+
+      if (!is.null(x$subset))
+         psize <- psize[x$subset]
+
+   }
+
+   if (length(colout) == 1L)
+      colout <- rep(colout, x$k.all)
+
+   if (length(colout) != x$k.all)
+      stop(mstyle$stop(paste0("Length of the 'colout' argument (", length(colout), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
+
+   if (!is.null(x$subset))
+      colout <- colout[x$subset]
 
    ### extract fitted values
 
-   options(na.action = "na.pass")               ### using na.pass to get the entire vector (length of yi.f)
+   options(na.action = "na.pass") # using na.pass to get the entire vector (length of yi.f)
 
       if (x$int.only) {
          pred <- fitted(x)
          pred.ci.lb <- rep(NA_real_, k)
          pred.ci.ub <- rep(NA_real_, k)
       } else {
-         temp <- predict(x, level=level)
+         temp <- predict(x, level=level, pi.type=pi.type)
          pred <- temp$pred
-         if (addcred) {
-            pred.ci.lb <- temp$cr.lb
-            pred.ci.ub <- temp$cr.ub
+         if (addpred) {
+            pred.ci.lb <- temp$pi.lb
+            pred.ci.ub <- temp$pi.ub
          } else {
             pred.ci.lb <- temp$ci.lb
             pred.ci.ub <- temp$ci.ub
          }
       }
 
-      if (inherits(x, "rma.glmm")) {            ### TODO: change this when there is a weights() function for 'rma.glmm' objects
-         #weights <- NULL
+      weights <- try(weights(x), silent=TRUE) # does not work for rma.glmm and rma.uni.selmodel objects
+
+      if (inherits(weights, "try-error"))
          weights <- rep(1, k)
-      } else {
-         weights <- weights(x)                  ### these are the weights used for the actual model fitting
-      }
-
-   options(na.action = na.act)
-
-   ### if user has set the point sizes
-
-   if (!is.null(psize)) {                       ### note: psize must have same length as yi.f (including NAs)
-      if (length(psize) == 1L)                  ### or be equal to a single value (which is then repeated)
-         psize <- rep(psize, k)
-      if (length(psize) != length(yi))
-         stop(mstyle$stop("Number of outcomes does not correspond to the length of the 'psize' argument."))
-   }
 
    ### sort the data if requested
 
    if (!is.null(order)) {
 
-      if (is.character(order)) {
+      if (length(order) == 1L) {
 
-         if (length(order) != 1L)
-            stop(mstyle$stop("Incorrect length of 'order' argument."))
+         order <- match.arg(order, c("obs", "yi", "fit", "prec", "vi", "resid", "rstandard", "abs.resid", "abs.rstandard"))
 
-         if (order == "obs")
+         if (order == "obs" || order == "yi")
             sort.vec <- order(yi)
          if (order == "fit")
             sort.vec <- order(pred)
-         if (order == "prec")
+         if (order == "prec" || order == "vi")
             sort.vec <- order(vi, yi)
          if (order == "resid")
             sort.vec <- order(yi-pred, yi)
          if (order == "rstandard")
-            sort.vec <- order(rstandard(x)$z, yi)
+            sort.vec <- order(rstandard(x)$z, yi)      # need options(na.action = "na.pass") here as well
          if (order == "abs.resid")
             sort.vec <- order(abs(yi-pred), yi)
          if (order == "abs.rstandard")
-            sort.vec <- order(abs(rstandard(x)$z), yi)
+            sort.vec <- order(abs(rstandard(x)$z), yi) # need options(na.action = "na.pass") here as well
 
       } else {
-         sort.vec <- order                      ### in principle, can also subset with the order argument
+
+         if (length(order) != x$k.all)
+            stop(mstyle$stop(paste0("Length of the 'order' argument (", length(order), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
+
+         if (grepl("^order\\(", deparse(substitute(order)))) {
+            sort.vec <- order
+         } else {
+            sort.vec <- order(order, decreasing=decreasing)
+         }
+
+         if (!is.null(x$subset))
+            sort.vec <- sort.vec[x$subset] - sum(!x$subset)
+
       }
 
       yi         <- yi[sort.vec]
       vi         <- vi[sort.vec]
       X          <- X[sort.vec,,drop=FALSE]
       slab       <- slab[sort.vec]
-      ilab       <- ilab[sort.vec,,drop=FALSE]  ### if ilab is still NULL, then this remains NULL
+      ilab       <- ilab[sort.vec,,drop=FALSE]  # if NULL, remains NULL
       pred       <- pred[sort.vec]
       pred.ci.lb <- pred.ci.lb[sort.vec]
       pred.ci.ub <- pred.ci.ub[sort.vec]
       weights    <- weights[sort.vec]
       pch        <- pch[sort.vec]
-      psize      <- psize[sort.vec]             ### if psize is still NULL, then this remains NULL
+      psize      <- psize[sort.vec]             # if NULL, remains NULL
+      colout     <- colout[sort.vec]
 
    }
 
-   k <- length(yi)                              ### in case length of k has changed
+   options(na.action = na.act)
+
+   k <- length(yi)                              # in case length of k has changed
 
    ### set rows value
 
    if (missing(rows)) {
       rows <- k:1
    } else {
-      if (length(rows) == 1L) {                 ### note: rows must be a single value or the same
-         rows <- rows:(rows-k+1)                ### length of yi.f (including NAs) *after ordering/subsetting*
+      if (length(rows) == 1L) {                 # note: rows must be a single value or the same
+         rows <- rows:(rows-k+1)                # length of yi.f (including NAs) *after ordering*
       }
    }
 
-   if (length(rows) != length(yi))
-      stop(mstyle$stop("Number of outcomes does not correspond to the length of the 'rows' argument."))
+   if (length(rows) != k)
+      stop(mstyle$stop(paste0("Length of the 'rows' argument (", length(rows), ") does not correspond to the number of outcomes (", k, ").")))
 
    ### reverse order
 
@@ -304,16 +381,17 @@ cex, cex.lab, cex.axis, annosym, ...) {
    vi         <- vi[k:1]
    X          <- X[k:1,,drop=FALSE]
    slab       <- slab[k:1]
-   ilab       <- ilab[k:1,,drop=FALSE]          ### if ilab is still NULL, then this remains NULL
+   ilab       <- ilab[k:1,,drop=FALSE]          # if NULL, remains NULL
    pred       <- pred[k:1]
    pred.ci.lb <- pred.ci.lb[k:1]
    pred.ci.ub <- pred.ci.ub[k:1]
    weights    <- weights[k:1]
    pch        <- pch[k:1]
-   psize      <- psize[k:1]                     ### if psize is still NULL, then this remains NULL
+   psize      <- psize[k:1]                     # if NULL, remains NULL
+   colout     <- colout[k:1]
    rows       <- rows[k:1]
 
-   ### check for NAs in yi/vi and act accordingly
+   ### check for NAs in yi/vi/X and act accordingly
 
    yiviX.na <- is.na(yi) | is.na(vi) | apply(is.na(X), 1, any)
 
@@ -326,16 +404,17 @@ cex, cex.lab, cex.axis, annosym, ...) {
          vi         <- vi[not.na]
          X          <- X[not.na,,drop=FALSE]
          slab       <- slab[not.na]
-         ilab       <- ilab[not.na,,drop=FALSE] ### if ilab is still NULL, then this remains NULL
+         ilab       <- ilab[not.na,,drop=FALSE] # if NULL, remains NULL
          pred       <- pred[not.na]
          pred.ci.lb <- pred.ci.lb[not.na]
          pred.ci.ub <- pred.ci.ub[not.na]
          weights    <- weights[not.na]
          pch        <- pch[not.na]
-         psize      <- psize[not.na]            ### if psize is still NULL, then this remains NULL
+         psize      <- psize[not.na]            # if NULL, remains NULL
+         colout     <- colout[not.na]
 
-         rows.new <- rows                       ### rearrange rows due to NAs being omitted from plot
-         rows.na  <- rows[!not.na]              ### shift higher rows down according to number of NAs omitted
+         rows.new <- rows                       # rearrange rows due to NAs being omitted from plot
+         rows.na  <- rows[!not.na]              # shift higher rows down according to number of NAs omitted
          for (j in seq_len(length(rows.na))) {
             rows.new[rows >= rows.na[j]] <- rows.new[rows >= rows.na[j]] - 1
          }
@@ -346,9 +425,9 @@ cex, cex.lab, cex.axis, annosym, ...) {
       if (na.act == "na.fail")
          stop(mstyle$stop("Missing values in results."))
 
-   }                                            ### note: yi/vi may be NA if na.act == "na.exclude" or "na.pass"
+   }                                            # note: yi/vi may be NA if na.act == "na.exclude" or "na.pass"
 
-   k <- length(yi)                              ### in case length of k has changed
+   k <- length(yi)                              # in case length of k has changed
 
    ### calculate individual CI bounds
 
@@ -385,45 +464,47 @@ cex, cex.lab, cex.axis, annosym, ...) {
    pred.ci.lb <- tmp[,1]
    pred.ci.ub <- tmp[,2]
 
-   ### apply ci limits if specified
+   ### apply observation/outcome limits if specified
 
-   if (!missing(clim)) {
-      clim <- sort(clim)
-      if (length(clim) != 2L)
-         stop(mstyle$stop("Argument 'clim' must be of length 2."))
-      ci.lb[ci.lb < clim[1]] <- clim[1]
-      ci.ub[ci.ub > clim[2]] <- clim[2]
-      pred.ci.lb[pred.ci.lb < clim[1]] <- clim[1]
-      pred.ci.ub[pred.ci.ub > clim[2]] <- clim[2]
+   if (!missing(olim)) {
+      if (length(olim) != 2L)
+         stop(mstyle$stop("Argument 'olim' must be of length 2."))
+      olim <- sort(olim)
+      yi[yi < olim[1]] <- olim[1]
+      yi[yi > olim[2]] <- olim[2]
+      ci.lb[ci.lb < olim[1]] <- olim[1]
+      ci.ub[ci.ub > olim[2]] <- olim[2]
+      pred.ci.lb[pred.ci.lb < olim[1]] <- olim[1]
+      pred.ci.ub[pred.ci.ub > olim[2]] <- olim[2]
    }
 
    ### set default point sizes (if not specified by user)
 
    if (is.null(psize)) {
-      # if (is.null(weights)) {
-      #    if (any(vi <= 0, na.rm=TRUE)) {           ### in case any vi value is zero
-      #       psize <- rep(1, k)
-      #    } else {                                  ### default psize is proportional to inverse standard error
-      #       wi    <- 1/sqrt(vi)                    ### note: vi's that are NA are ignored (but vi's whose yi is
-      #       psize <- wi/sum(wi, na.rm=TRUE)        ### NA are NOT ignored; an unlikely case in practice)
-      #       psize <- (psize - min(psize, na.rm=TRUE)) / (max(psize, na.rm=TRUE) - min(psize, na.rm=TRUE))
-      #       psize <- (psize * 1.0) + 0.5           ### note: only vi's that are still in the subset are used for determining the default point sizes
-      #       if (all(is.na(psize)))                 ### if k=1, then psize is NA, so catch this (and maybe some other problems)
-      #          psize <- rep(1, k)
-      #    }
-      # } else {
-      wi    <- weights
-      psize <- wi/sum(wi, na.rm=TRUE)
-      rng   <- max(psize, na.rm=TRUE) - min(psize, na.rm=TRUE)
-      if (rng <= .Machine$double.eps^0.5) {
-         psize <- rep(1, k)
-      } else {
-         psize <- (psize - min(psize, na.rm=TRUE)) / rng
-         psize <- (psize * 1.0) + 0.5
+      if (length(plim) < 2L)
+         stop(mstyle$stop("Argument 'plim' must be of length 2 or 3."))
+      wi <- sqrt(weights)
+      if (!is.na(plim[1]) && !is.na(plim[2])) {
+         rng <- max(wi, na.rm=TRUE) - min(wi, na.rm=TRUE)
+         if (rng <= .Machine$double.eps^0.5) {
+            psize <- rep(1, k)
+         } else {
+            psize <- (wi - min(wi, na.rm=TRUE)) / rng
+            psize <- (psize * (plim[2] - plim[1])) + plim[1]
+         }
+      }
+      if (is.na(plim[1]) && !is.na(plim[2])) {
+         psize <- wi / max(wi, na.rm=TRUE) * plim[2]
+         if (length(plim) == 3L)
+            psize[psize <= plim[3]] <- plim[3]
+      }
+      if (!is.na(plim[1]) && is.na(plim[2])) {
+         psize <- wi / min(wi, na.rm=TRUE) * plim[1]
+         if (length(plim) == 3L)
+            psize[psize >= plim[3]] <- plim[3]
       }
       if (all(is.na(psize)))
          psize <- rep(1, k)
-      # }
    }
 
    #########################################################################
@@ -454,7 +535,7 @@ cex, cex.lab, cex.axis, annosym, ...) {
       #xlim[2] <- xlim[2]*max(1, digits[[2]]/2)
    }
 
-   ### set x axis limits (at argument overrides alim argument)
+   ### set x-axis limits (at argument overrides alim argument)
 
    alim.spec <- TRUE
 
@@ -467,7 +548,7 @@ cex, cex.lab, cex.axis, annosym, ...) {
       }
    }
 
-   ### make sure the plot and x axis limits are sorted
+   ### make sure the plot and x-axis limits are sorted
 
    alim <- sort(alim)
    xlim <- sort(xlim)
@@ -477,12 +558,12 @@ cex, cex.lab, cex.axis, annosym, ...) {
    if (xlim[1] > min(yi, na.rm=TRUE)) { xlim[1] <- min(yi, na.rm=TRUE) }
    if (xlim[2] < max(yi, na.rm=TRUE)) { xlim[2] <- max(yi, na.rm=TRUE) }
 
-   ### x axis limits must always encompass the yi values (no longer required)
+   ### x-axis limits must always encompass the yi values (no longer required)
 
    #if (alim[1] > min(yi, na.rm=TRUE)) { alim[1] <- min(yi, na.rm=TRUE) }
    #if (alim[2] < max(yi, na.rm=TRUE)) { alim[2] <- max(yi, na.rm=TRUE) }
 
-   ### plot limits must always encompass the x axis limits
+   ### plot limits must always encompass the x-axis limits
 
    if (alim[1] < xlim[1]) { xlim[1] <- alim[1] }
    if (alim[2] > xlim[2]) { xlim[2] <- alim[2] }
@@ -501,19 +582,19 @@ cex, cex.lab, cex.axis, annosym, ...) {
    if (is.na(ddd$textpos[2]))
       ddd$textpos[2] <- xlim[2]
 
-   ### set y axis limits
+   ### set y-axis limits
 
    if (missing(ylim)) {
       if (x$int.only && addfit) {
-         ylim <- c(-1.5, k+top)
+         ylim <- c(-1.5, max(rows, na.rm=TRUE)+top)
       } else {
-         ylim <- c(0.5, k+top)
+         ylim <- c(0.5, max(rows, na.rm=TRUE)+top)
       }
    } else {
       ylim <- sort(ylim)
    }
 
-   ### generate x axis positions if none are specified
+   ### generate x-axis positions if none are specified
 
    if (is.null(at)) {
       if (alim.spec) {
@@ -522,23 +603,23 @@ cex, cex.lab, cex.axis, annosym, ...) {
          at <- pretty(x=c(min(ci.lb, na.rm=TRUE), max(ci.ub, na.rm=TRUE)), n=steps-1)
       }
    } else {
-      at[at < alim[1]] <- alim[1] ### remove at values that are below or above the axis limits
+      at[at < alim[1]] <- alim[1] # remove at values that are below or above the axis limits
       at[at > alim[2]] <- alim[2]
       at <- unique(at)
    }
 
-   ### x axis labels (apply transformation to axis labels if requested)
+   ### x-axis labels (apply transformation to axis labels if requested)
 
    at.lab <- at
 
    if (is.function(atransf)) {
       if (is.null(targs)) {
-         at.lab <- formatC(sapply(at.lab, atransf), digits=digits[[2]], format="f", drop0trailing=ifelse(class(digits[[2]]) == "integer", TRUE, FALSE))
+         at.lab <- formatC(sapply(at.lab, atransf), digits=digits[[2]], format="f", drop0trailing=is.integer(digits[[2]]))
       } else {
-         at.lab <- formatC(sapply(at.lab, atransf, targs), digits=digits[[2]], format="f", drop0trailing=ifelse(class(digits[[2]]) == "integer", TRUE, FALSE))
+         at.lab <- formatC(sapply(at.lab, atransf, targs), digits=digits[[2]], format="f", drop0trailing=is.integer(digits[[2]]))
       }
    } else {
-      at.lab <- formatC(at.lab, digits=digits[[2]], format="f", drop0trailing=ifelse(class(digits[[2]]) == "integer", TRUE, FALSE))
+      at.lab <- formatC(at.lab, digits=digits[[2]], format="f", drop0trailing=is.integer(digits[[2]]))
    }
 
    #########################################################################
@@ -566,7 +647,7 @@ cex, cex.lab, cex.axis, annosym, ...) {
    par.mar.adj <- par.mar - c(0,3,1,1)
    par.mar.adj[par.mar.adj < 0] <- 0
    par(mar = par.mar.adj)
-   on.exit(par(mar = par.mar))
+   on.exit(par(mar = par.mar), add=TRUE)
 
    ### start plot
 
@@ -576,14 +657,17 @@ cex, cex.lab, cex.axis, annosym, ...) {
 
    labline(h=ylim[2]-(top-1), lty=lty[3], ...)
 
+   ### get coordinates of the plotting region
+
+   par.usr <- par("usr")
+
    ### add reference line
 
    if (is.numeric(refline))
-      lsegments(refline, ylim[1]-5, refline, ylim[2]-(top-1), lty="dotted", ...)
+      lsegments(refline, par.usr[3], refline, ylim[2]-(top-1), lty="dotted", ...)
 
    ### set cex, cex.lab, and cex.axis sizes as a function of the height of the figure
 
-   par.usr <- par("usr")
    height  <- par.usr[4] - par.usr[3]
 
    if (is.null(cex)) {
@@ -633,48 +717,48 @@ cex, cex.lab, cex.axis, annosym, ...) {
 
       if (inherits(x, "rma.mv") && x$withG && x$tau2s > 1) {
 
-         if (!is.logical(addcred)) {
-            ### for multiple tau^2 (and gamma^2) values, need to specify level(s) of the inner factor(s) to compute the credibility interval
-            ### this can be done via the addcred argument (i.e., instead of using a logical, one specifies the level(s))
-            if (length(addcred) == 1L)
-               addcred <- c(addcred, addcred)
-            temp <- predict(x, level=level, tau2.levels=addcred[1], gamma2.levels=addcred[2])
-            addcred <- TRUE ### set addcred to TRUE, so if (x$method != "FE" && addcred) further below works
+         if (!is.logical(addpred)) {
+            ### for multiple tau^2 (and gamma^2) values, need to specify level(s) of the inner factor(s) to compute the prediction interval
+            ### this can be done via the addpred argument (i.e., instead of using a logical, one specifies the level(s))
+            if (length(addpred) == 1L)
+               addpred <- c(addpred, addpred)
+            temp <- predict(x, level=level, tau2.levels=addpred[1], gamma2.levels=addpred[2], pi.type=pi.type)
+            addpred <- TRUE ### set addpred to TRUE, so if (!is.element(x$method, c("FE","EE","CE")) && addpred) further below works
          } else {
-            if (addcred) {
-               ### here addcred=TRUE, but user has not specified the level, so throw an error
-               stop(mstyle$stop("Need to specify the level of the inner factor(s) via the 'addcred' argument."))
+            if (addpred) {
+               ### here addpred=TRUE, but user has not specified the level, so throw an error
+               stop(mstyle$stop("Must specify the level of the inner factor(s) via the 'addpred' argument."))
             } else {
-               ### here addcred=FALSE, so just use the first tau^2 and gamma^2 arbitrarily (so predict() works)
-               temp <- predict(x, level=level, tau2.levels=1, gamma2.levels=1)
+               ### here addpred=FALSE, so just use the first tau^2 and gamma^2 arbitrarily (so predict() works)
+               temp <- predict(x, level=level, tau2.levels=1, gamma2.levels=1, pi.type=pi.type)
             }
          }
 
       } else {
 
-         temp <- predict(x, level=level)
+         temp <- predict(x, level=level, pi.type=pi.type)
 
       }
 
       beta       <- temp$pred
       beta.ci.lb <- temp$ci.lb
       beta.ci.ub <- temp$ci.ub
-      beta.cr.lb <- temp$cr.lb
-      beta.cr.ub <- temp$cr.ub
+      beta.pi.lb <- temp$pi.lb
+      beta.pi.ub <- temp$pi.ub
 
       if (is.function(transf)) {
          if (is.null(targs)) {
             beta       <- sapply(beta, transf)
             beta.ci.lb <- sapply(beta.ci.lb, transf)
             beta.ci.ub <- sapply(beta.ci.ub, transf)
-            beta.cr.lb <- sapply(beta.cr.lb, transf)
-            beta.cr.ub <- sapply(beta.cr.ub, transf)
+            beta.pi.lb <- sapply(beta.pi.lb, transf)
+            beta.pi.ub <- sapply(beta.pi.ub, transf)
          } else {
             beta       <- sapply(beta, transf, targs)
             beta.ci.lb <- sapply(beta.ci.lb, transf, targs)
             beta.ci.ub <- sapply(beta.ci.ub, transf, targs)
-            beta.cr.lb <- sapply(beta.cr.lb, transf, targs)
-            beta.cr.ub <- sapply(beta.cr.ub, transf, targs)
+            beta.pi.lb <- sapply(beta.pi.lb, transf, targs)
+            beta.pi.ub <- sapply(beta.pi.ub, transf, targs)
          }
       }
 
@@ -684,33 +768,35 @@ cex, cex.lab, cex.axis, annosym, ...) {
       beta.ci.lb <- tmp[,1]
       beta.ci.ub <- tmp[,2]
 
-      tmp <- .psort(beta.cr.lb, beta.cr.ub)
-      beta.cr.lb <- tmp[,1]
-      beta.cr.ub <- tmp[,2]
+      tmp <- .psort(beta.pi.lb, beta.pi.ub)
+      beta.pi.lb <- tmp[,1]
+      beta.pi.ub <- tmp[,2]
 
-      ### apply ci limits if specified
+      ### apply observation/outcome limits if specified
 
-      if (!missing(clim)) {
-         beta.ci.lb[beta.ci.lb < clim[1]] <- clim[1]
-         beta.ci.ub[beta.ci.ub > clim[2]] <- clim[2]
-         beta.cr.lb[beta.cr.lb < clim[1]] <- clim[1]
-         beta.cr.ub[beta.cr.ub > clim[2]] <- clim[2]
+      if (!missing(olim)) {
+         pred[pred < olim[1]] <- olim[1]
+         pred[pred > olim[2]] <- olim[2]
+         beta.ci.lb[beta.ci.lb < olim[1]] <- olim[1]
+         beta.ci.ub[beta.ci.ub > olim[2]] <- olim[2]
+         beta.pi.lb[beta.pi.lb < olim[1]] <- olim[1]
+         beta.pi.ub[beta.pi.ub > olim[2]] <- olim[2]
       }
 
-      ### add credibility interval
+      ### add prediction interval
 
-      if (x$method != "FE" && addcred) {
+      if (!is.element(x$method, c("FE","EE","CE")) && addpred) {
 
-         lsegments(max(beta.cr.lb, alim[1]), -1, min(beta.cr.ub, alim[2]), -1, lty=lty[2], col=col[2], ...)
+         lsegments(max(beta.pi.lb, alim[1]), -1, min(beta.pi.ub, alim[2]), -1, lty=lty[2], col=col[2], ...)
 
-         if (beta.cr.lb >= alim[1]) {
-            lsegments(beta.cr.lb, -1-(height/150)*cex*efac[1], beta.cr.lb, -1+(height/150)*cex*efac[1], col=col[2], ...)
+         if (beta.pi.lb >= alim[1]) {
+            lsegments(beta.pi.lb, -1-(height/150)*cex*efac[1], beta.pi.lb, -1+(height/150)*cex*efac[1], col=col[2], ...)
          } else {
             lpolygon(x=c(alim[1], alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]), y=c(-1, -1+(height/150)*cex*efac[2], -1-(height/150)*cex*efac[2], -1), col=col[2], border=col[2], ...)
          }
 
-         if (beta.cr.ub <= alim[2]) {
-            lsegments(beta.cr.ub, -1-(height/150)*cex*efac[1], beta.cr.ub, -1+(height/150)*cex*efac[1], col=col[2], ...)
+         if (beta.pi.ub <= alim[2]) {
+            lsegments(beta.pi.ub, -1-(height/150)*cex*efac[1], beta.pi.ub, -1+(height/150)*cex*efac[1], col=col[2], ...)
          } else {
             lpolygon(x=c(alim[2], alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]), y=c(-1, -1+(height/150)*cex*efac[2], -1-(height/150)*cex*efac[2], -1), col=col[2], border=col[2], ...)
          }
@@ -724,19 +810,23 @@ cex, cex.lab, cex.axis, annosym, ...) {
       ### add label for model estimate
 
       if (missing(mlab))
-         mlab <- ifelse((x$method=="FE"), "FE Model", "RE Model")
+         mlab <- sapply(x$method, switch, "FE"="FE Model", "EE"="EE Model", "CE"="CE Model", "RE Model", USE.NAMES=FALSE)
 
-      ltext(ddd$textpos[1], -1, mlab, pos=4, cex=cex, ...)
+      if (is.list(mlab)) {
+         ltext(ddd$textpos[1], -1, mlab[[1]], pos=4, cex=cex, ...)
+      } else {
+         ltext(ddd$textpos[1], -1, mlab, pos=4, cex=cex, ...)
+      }
 
    }
 
    #########################################################################
 
-   ### add x axis
+   ### add x-axis
 
    laxis(side=1, at=at, labels=at.lab, cex.axis=cex.axis, ...)
 
-   ### add x axis label
+   ### add x-axis label
 
    if (missing(xlab))
       xlab <- .setlab(measure, transf.char, atransf.char, gentype=1)
@@ -747,34 +837,34 @@ cex, cex.lab, cex.axis, annosym, ...) {
 
    for (i in seq_len(k)) {
 
-      ### need to skip missings, as if() check below will otherwise throw an error
+      ### need to skip missings (if check below will otherwise throw an error)
       if (is.na(yi[i]) || is.na(vi[i]))
          next
 
       ### if the lower bound is actually larger than upper x-axis limit, then everything is to the right and just draw a polygon pointing in that direction
       if (ci.lb[i] >= alim[2]) {
-         lpolygon(x=c(alim[2], alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]), y=c(rows[i], rows[i]+(height/150)*cex*efac[2], rows[i]-(height/150)*cex*efac[2], rows[i]), col="black", ...)
+         lpolygon(x=c(alim[2], alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]), y=c(rows[i], rows[i]+(height/150)*cex*efac[2], rows[i]-(height/150)*cex*efac[2], rows[i]), col=colout[i], ...)
          next
       }
 
       ### if the upper bound is actually lower than lower x-axis limit, then everything is to the left and just draw a polygon pointing in that direction
       if (ci.ub[i] <= alim[1]) {
-         lpolygon(x=c(alim[1], alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]), y=c(rows[i], rows[i]+(height/150)*cex*efac[2], rows[i]-(height/150)*cex*efac[2], rows[i]), col="black", ...)
+         lpolygon(x=c(alim[1], alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]), y=c(rows[i], rows[i]+(height/150)*cex*efac[2], rows[i]-(height/150)*cex*efac[2], rows[i]), col=colout[i], ...)
          next
       }
 
-      lsegments(max(ci.lb[i], alim[1]), rows[i], min(ci.ub[i], alim[2]), rows[i], lty=lty[1], ...)
+      lsegments(max(ci.lb[i], alim[1]), rows[i], min(ci.ub[i], alim[2]), rows[i], lty=lty[1], col=colout[i], ...)
 
       if (ci.lb[i] >= alim[1]) {
-         lsegments(ci.lb[i], rows[i]-(height/150)*cex*efac[1], ci.lb[i], rows[i]+(height/150)*cex*efac[1], ...)
+         lsegments(ci.lb[i], rows[i]-(height/150)*cex*efac[1], ci.lb[i], rows[i]+(height/150)*cex*efac[1], col=colout[i], ...)
       } else {
-         lpolygon(x=c(alim[1], alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]), y=c(rows[i], rows[i]+(height/150)*cex*efac[2], rows[i]-(height/150)*cex*efac[2], rows[i]), col="black", ...)
+         lpolygon(x=c(alim[1], alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]), y=c(rows[i], rows[i]+(height/150)*cex*efac[2], rows[i]-(height/150)*cex*efac[2], rows[i]), col=colout[i], ...)
       }
 
       if (ci.ub[i] <= alim[2]) {
-         lsegments(ci.ub[i], rows[i]-(height/150)*cex*efac[1], ci.ub[i], rows[i]+(height/150)*cex*efac[1], ...)
+         lsegments(ci.ub[i], rows[i]-(height/150)*cex*efac[1], ci.ub[i], rows[i]+(height/150)*cex*efac[1], col=colout[i], ...)
       } else {
-         lpolygon(x=c(alim[2], alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]), y=c(rows[i], rows[i]+(height/150)*cex*efac[2], rows[i]-(height/150)*cex*efac[2], rows[i]), col="black", ...)
+         lpolygon(x=c(alim[2], alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]), y=c(rows[i], rows[i]+(height/150)*cex*efac[2], rows[i]-(height/150)*cex*efac[2], rows[i]), col=colout[i], ...)
       }
 
    }
@@ -844,7 +934,8 @@ cex, cex.lab, cex.axis, annosym, ...) {
          }
       }
 
-      annotext <- formatC(annotext, format="f", digits=digits[[1]])
+      annotext <- .fcf(annotext, digits[[1]])
+      annotext <- sub("-", annosym[4], annotext, fixed=TRUE)
 
       if (missing(width)) {
          width <- apply(annotext, 2, function(x) max(nchar(x)))
@@ -880,16 +971,16 @@ cex, cex.lab, cex.axis, annosym, ...) {
 
    for (i in seq_len(k)) {
 
-      ### need to skip missings, as if() check below will otherwise throw an error
+      ### need to skip missings, as if () check below will otherwise throw an error
       if (is.na(yi[i]))
          next
 
       if (yi[i] >= alim[1] && yi[i] <= alim[2])
-         lpoints(yi[i], rows[i], pch=pch[i], cex=cex*psize[i], ...)
+         lpoints(x=yi[i], y=rows[i], pch=pch[i], col=colout[i], cex=cex*psize[i], ...)
 
    }
 
-   #lpoints(yi, rows, pch=pch, cex=cex*psize, ...)
+   #lpoints(x=yi, y=rows, pch=pch, cex=cex*psize, ...)
 
    ### add horizontal line at 0 for the standard FE/RE model display
 
@@ -898,14 +989,14 @@ cex, cex.lab, cex.axis, annosym, ...) {
 
    ### add header
 
-   ltext(ddd$textpos[1], ylim[2]-(top-1)+1, header.left, pos=4, font=2, cex=cex, ...)
+   ltext(ddd$textpos[1], ylim[2]-(top-1)+1, header.left,  pos=4, font=2, cex=cex, ...)
    ltext(ddd$textpos[2], ylim[2]-(top-1)+1, header.right, pos=2, font=2, cex=cex, ...)
 
    #########################################################################
 
    ### return some information about plot invisibly
 
-   res <- list('xlim'=par("usr")[1:2], 'alim'=alim, 'at'=at, 'ylim'=ylim, 'rows'=rows, 'cex'=cex, 'cex.lab'=cex.lab, 'cex.axis'=cex.axis)
+   res <- list(xlim=par("usr")[1:2], alim=alim, at=at, ylim=ylim, rows=rows, cex=cex, cex.lab=cex.lab, cex.axis=cex.axis)
 
    invisible(res)
 
