@@ -1,7 +1,8 @@
 nobs.rma <- function(object, ...) {
 
-   if (!inherits(object, "rma"))
-      stop("Argument 'object' must be an object of class \"rma\".")
+   mstyle <- .get.mstyle("crayon" %in% .packages())
+
+   .chkclass(class(object), must="rma")
 
    n.obs <- object$k.eff - ifelse(object$method == "REML", 1, 0) * object$p.eff
 
