@@ -4,8 +4,7 @@ col.na="lightgray", ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (!inherits(x, "infl.rma.uni"))
-      stop(mstyle$stop("Argument 'x' must be an object of class \"infl.rma.uni\"."))
+   .chkclass(class(x), must="infl.rma.uni")
 
    na.act <- getOption("na.action")
 
@@ -18,12 +17,8 @@ col.na="lightgray", ...) {
 
    any.na <- is.na(as.data.frame(x$inf))
 
-   if (any(any.na)) {
-
-      if (na.act == "na.fail")
-         stop(mstyle$stop("Missing values in results."))
-
-   }
+   if (any(any.na) && na.act == "na.fail")
+      stop(mstyle$stop("Missing values in results."))
 
    #########################################################################
 
@@ -73,14 +68,13 @@ col.na="lightgray", ...) {
 
    #########################################################################
 
-   ### filter out potential arguments for abbreviate() (which would cause problems with the various plot functions)
-   ### see: http://ucfagls.wordpress.com/2011/07/23/passing-non-graphical-parameters-to-graphical-functions-using/
+   ### filter out potential arguments to abbreviate() (which cause problems with the various plot functions)
 
-   lplot   <- function(..., minlength, strict) { plot(...) }
-   lpoints <- function(..., minlength, strict) { points(...) }
-   llines  <- function(..., minlength, strict) { lines(...) }
-   laxis   <- function(..., minlength, strict) { axis(...) }
-   labline <- function(..., minlength, strict) { abline(...) }
+   lplot   <- function(..., minlength, strict) plot(...)
+   lpoints <- function(..., minlength, strict) points(...)
+   llines  <- function(..., minlength, strict) lines(...)
+   laxis   <- function(..., minlength, strict) axis(...)
+   labline <- function(..., minlength, strict) abline(...)
 
    #########################################################################
 
@@ -94,31 +88,32 @@ col.na="lightgray", ...) {
    if (plotinf) {
 
       ### set layout (either defaults or user-specified)
+      ### note: could also use n2mfrow() here, but this behaves slightly differently
 
       par.mfrow <- par("mfrow")
       on.exit(par(mfrow = par.mfrow), add=TRUE)
 
       if (missing(layout)) {
 
-         if (length(which.inf) == 2)
+         if (length(which.inf) == 2L)
             par(mfrow=c(2,1))
 
-         if (length(which.inf) == 3)
+         if (length(which.inf) == 3L)
             par(mfrow=c(3,1))
 
-         if (length(which.inf) == 4)
+         if (length(which.inf) == 4L)
             par(mfrow=c(2,2))
 
-         if (length(which.inf) == 5)
+         if (length(which.inf) == 5L)
             par(mfrow=c(5,1))
 
-         if (length(which.inf) == 6)
+         if (length(which.inf) == 6L)
             par(mfrow=c(3,2))
 
-         if (length(which.inf) == 7)
+         if (length(which.inf) == 7L)
             par(mfrow=c(7,1))
 
-         if (length(which.inf) == 8)
+         if (length(which.inf) == 8L)
             par(mfrow=c(4,2))
 
       } else {
@@ -159,8 +154,8 @@ col.na="lightgray", ...) {
                if (na.act == "na.exclude" || na.act == "na.pass")
                   llines(seq_len(len.ids)[not.na], zi[not.na], col=col.na, ...)
                llines(seq_len(len.ids), zi, ...)
-               lpoints(seq_len(len.ids), zi, pch=pch, bg=bg, ...)
-               lpoints(seq_len(len.ids)[ids.infl], zi[ids.infl], bg=bg.infl, pch=pch, ...)
+               lpoints(x=seq_len(len.ids),           y=zi,           bg=bg,      pch=pch, ...)
+               lpoints(x=seq_len(len.ids)[ids.infl], y=zi[ids.infl], bg=bg.infl, pch=pch, ...)
                #if (num.infl)
                #   text(seq_len(len.ids)[ids.infl], zi[ids.infl], seq_len(len.ids)[ids.infl], pos=ifelse(zi[ids.infl] > 0, 3, 1), ...)
             } else {
@@ -196,8 +191,8 @@ col.na="lightgray", ...) {
                if (na.act == "na.exclude" || na.act == "na.pass")
                   llines(seq_len(len.ids)[not.na], zi[not.na], col=col.na, ...)
                llines(seq_len(len.ids), zi, ...)
-               lpoints(seq_len(len.ids), zi, pch=pch, bg=bg, ...)
-               lpoints(seq_len(len.ids)[ids.infl], zi[ids.infl], bg=bg.infl, pch=pch, ...)
+               lpoints(x=seq_len(len.ids),           y=zi,           bg=bg,      pch=pch, ...)
+               lpoints(x=seq_len(len.ids)[ids.infl], y=zi[ids.infl], bg=bg.infl, pch=pch, ...)
                #if (num.infl)
                #   text(seq_len(len.ids)[ids.infl], zi[ids.infl], seq_len(len.ids)[ids.infl], pos=ifelse(zi[ids.infl] > 0, 3, 1), ...)
             } else {
@@ -231,8 +226,8 @@ col.na="lightgray", ...) {
                if (na.act == "na.exclude" || na.act == "na.pass")
                   llines(seq_len(len.ids)[not.na], zi[not.na], col=col.na, ...)
                llines(seq_len(len.ids), zi, ...)
-               lpoints(seq_len(len.ids), zi, pch=pch, bg=bg, ...)
-               lpoints(seq_len(len.ids)[ids.infl], zi[ids.infl], bg=bg.infl, pch=pch, ...)
+               lpoints(x=seq_len(len.ids),           y=zi,           bg=bg,      pch=pch, ...)
+               lpoints(x=seq_len(len.ids)[ids.infl], y=zi[ids.infl], bg=bg.infl, pch=pch, ...)
                #if (num.infl)
                #   text(seq_len(len.ids)[ids.infl], zi[ids.infl], seq_len(len.ids)[ids.infl], pos=3, ...)
             } else {
@@ -272,8 +267,8 @@ col.na="lightgray", ...) {
                if (na.act == "na.exclude" || na.act == "na.pass")
                   llines(seq_len(len.ids)[not.na], zi[not.na], col=col.na, ...)
                llines(seq_len(len.ids), zi, ...)
-               lpoints(seq_len(len.ids), zi, pch=pch, bg=bg, ...)
-               lpoints(seq_len(len.ids)[ids.infl], zi[ids.infl], bg=bg.infl, pch=pch, ...)
+               lpoints(x=seq_len(len.ids),           y=zi,           bg=bg,      pch=pch, ...)
+               lpoints(x=seq_len(len.ids)[ids.infl], y=zi[ids.infl], bg=bg.infl, pch=pch, ...)
                #if (num.infl)
                #   text(seq_len(len.ids)[ids.infl], zi[ids.infl], seq_len(len.ids)[ids.infl], pos=ifelse(zi[ids.infl] > 1, 3, 1), ...)
             } else {
@@ -307,8 +302,8 @@ col.na="lightgray", ...) {
                if (na.act == "na.exclude" || na.act == "na.pass")
                   llines(seq_len(len.ids)[not.na], zi[not.na], col=col.na, ...)
                llines(seq_len(len.ids), zi, ...)
-               lpoints(seq_len(len.ids), zi, pch=pch, bg=bg, ...)
-               lpoints(seq_len(len.ids)[ids.infl], zi[ids.infl], bg=bg.infl, pch=pch, ...)
+               lpoints(x=seq_len(len.ids),           y=zi,           bg=bg,      pch=pch, ...)
+               lpoints(x=seq_len(len.ids)[ids.infl], y=zi[ids.infl], bg=bg.infl, pch=pch, ...)
             } else {
                lplot(NA, NA, xlim=c(0,1), ylim=c(0,1), xaxt="n", yaxt="n", main="tau2.del", xlab="", ylab="", ...)
             }
@@ -341,8 +336,8 @@ col.na="lightgray", ...) {
                if (na.act == "na.exclude" || na.act == "na.pass")
                   llines(seq_len(len.ids)[not.na], zi[not.na], col=col.na, ...)
                llines(seq_len(len.ids), zi, ...)
-               lpoints(seq_len(len.ids), zi, pch=pch, bg=bg, ...)
-               lpoints(seq_len(len.ids)[ids.infl], zi[ids.infl], bg=bg.infl, pch=pch, ...)
+               lpoints(x=seq_len(len.ids),           y=zi,           bg=bg,      pch=pch, ...)
+               lpoints(x=seq_len(len.ids)[ids.infl], y=zi[ids.infl], bg=bg.infl, pch=pch, ...)
             } else {
                lplot(NA, NA, xlim=c(0,1), ylim=c(0,1), xaxt="n", yaxt="n", main="QE.del", xlab="", ylab="", ...)
             }
@@ -375,8 +370,8 @@ col.na="lightgray", ...) {
                if (na.act == "na.exclude" || na.act == "na.pass")
                   llines(seq_len(len.ids)[not.na], zi[not.na], col=col.na, ...)
                llines(seq_len(len.ids), zi, ...)
-               lpoints(seq_len(len.ids), zi, pch=pch, bg=bg, ...)
-               lpoints(seq_len(len.ids)[ids.infl], zi[ids.infl], bg=bg.infl, pch=pch, ...)
+               lpoints(x=seq_len(len.ids),           y=zi,           bg=bg,      pch=pch, ...)
+               lpoints(x=seq_len(len.ids)[ids.infl], y=zi[ids.infl], bg=bg.infl, pch=pch, ...)
             } else {
                lplot(NA, NA, xlim=c(0,1), ylim=c(0,1), xaxt="n", yaxt="n", main="hat", xlab="", ylab="", ...)
             }
@@ -408,8 +403,8 @@ col.na="lightgray", ...) {
                if (na.act == "na.exclude" || na.act == "na.pass")
                   llines(seq_len(len.ids)[not.na], zi[not.na], col=col.na, ...)
                llines(seq_len(len.ids), zi, ...)
-               lpoints(seq_len(len.ids), zi, pch=pch, bg=bg, ...)
-               lpoints(seq_len(len.ids)[ids.infl], zi[ids.infl], bg=bg.infl, pch=pch, ...)
+               lpoints(x=seq_len(len.ids),           y=zi,           bg=bg,      pch=pch, ...)
+               lpoints(x=seq_len(len.ids)[ids.infl], y=zi[ids.infl], bg=bg.infl, pch=pch, ...)
             } else {
                lplot(NA, NA, xlim=c(0,1), ylim=c(0,1), xaxt="n", yaxt="n", main="weight", xlab="", ylab="", ...)
             }
@@ -467,8 +462,8 @@ col.na="lightgray", ...) {
          if (na.act == "na.exclude" || na.act == "na.pass")
             llines(seq_len(len.ids)[not.na], zi[not.na], col=col.na, ...)
          llines(seq_len(len.ids), zi, ...)
-         lpoints(seq_len(len.ids), zi, pch=pch, bg=bg, ...)
-         lpoints(seq_len(len.ids)[ids.infl], zi[ids.infl], bg=bg.infl, pch=pch, ...)
+         lpoints(x=seq_len(len.ids),           y=zi,           bg=bg,      pch=pch, ...)
+         lpoints(x=seq_len(len.ids)[ids.infl], y=zi[ids.infl], bg=bg.infl, pch=pch, ...)
 
       }
 
