@@ -1,12 +1,13 @@
 weights.rma.mh <- function(object, type="diagonal", ...) {
 
-   if (!inherits(object, "rma.mh"))
-      stop("Argument 'object' must be an object of class \"rma.mh\".")
+   mstyle <- .get.mstyle("crayon" %in% .packages())
+
+   .chkclass(class(object), must="rma.mh")
 
    na.act <- getOption("na.action")
 
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
-      stop("Unknown 'na.action' specified under options().")
+      stop(mstyle$stop("Unknown 'na.action' specified under options()."))
 
    type <- match.arg(type, c("diagonal", "matrix"))
 
@@ -40,14 +41,14 @@ weights.rma.mh <- function(object, type="diagonal", ...) {
    if (type == "diagonal") {
 
       weight <- rep(NA_real_, x$k.f)
-      weight[x$not.na] <- wi/sum(wi) * 100
+      weight[x$not.na] <- wi / sum(wi) * 100
       names(weight) <- x$slab
 
       if (na.act == "na.omit")
          weight <- weight[x$not.na]
 
       if (na.act == "na.fail" && any(!x$not.na))
-         stop("Missing values in weights.")
+         stop(mstyle$stop("Missing values in weights."))
 
       return(weight)
 
@@ -65,7 +66,7 @@ weights.rma.mh <- function(object, type="diagonal", ...) {
          Wfull <- Wfull[x$not.na, x$not.na, drop=FALSE]
 
       if (na.act == "na.fail" && any(!x$not.na))
-         stop("Missing values in results.")
+         stop(mstyle$stop("Missing values in results."))
 
       return(Wfull)
 
