@@ -1,12 +1,13 @@
 weights.rma.uni <- function(object, type="diagonal", ...) {
 
-   if (!inherits(object, "rma.uni"))
-      stop("Argument 'object' must be an object of class \"rma.uni\".")
+   mstyle <- .get.mstyle("crayon" %in% .packages())
+
+   .chkclass(class(object), must="rma.uni", notav="rma.uni.selmodel")
 
    na.act <- getOption("na.action")
 
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
-      stop("Unknown 'na.action' specified under options().")
+      stop(mstyle$stop("Unknown 'na.action' specified under options()."))
 
    type <- match.arg(type, c("diagonal", "matrix"))
 
@@ -30,14 +31,14 @@ weights.rma.uni <- function(object, type="diagonal", ...) {
 
       wi <- as.vector(diag(W))
       weight <- rep(NA_real_, x$k.f)
-      weight[x$not.na] <- wi/sum(wi) * 100
+      weight[x$not.na] <- wi / sum(wi) * 100
       names(weight) <- x$slab
 
       if (na.act == "na.omit")
          weight <- weight[x$not.na]
 
       if (na.act == "na.fail" && any(!x$not.na))
-         stop("Missing values in weights.")
+         stop(mstyle$stop("Missing values in weights."))
 
       return(weight)
 
@@ -55,7 +56,7 @@ weights.rma.uni <- function(object, type="diagonal", ...) {
          Wfull <- Wfull[x$not.na, x$not.na, drop=FALSE]
 
       if (na.act == "na.fail" && any(!x$not.na))
-         stop("Missing values in results.")
+         stop(mstyle$stop("Missing values in results."))
 
       return(Wfull)
 
