@@ -1,11 +1,13 @@
 ### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true")
 
-### see also: http://www.metafor-project.org/doku.php/analyses:raudenbush2009
+### see also: https://www.metafor-project.org/doku.php/analyses:raudenbush2009
 
 context("Checking analysis example: raudenbush2009")
 
+source("tolerances.r") # read in tolerances
+
 ### load data
-dat <- get(data(dat.raudenbush1985, package="metafor"))
+dat <- dat.raudenbush1985
 
 test_that("results are correct for the fixed-effects model.", {
 
@@ -13,9 +15,9 @@ test_that("results are correct for the fixed-effects model.", {
    res.FE <- rma(yi, vi, data=dat, digits=3, method="FE")
 
    ### compare with results on page 301 (Table 16.2) and page 302
-   expect_equivalent(round(coef(res.FE),3), 0.060)
-   expect_equivalent(round(res.FE$QE,2), 35.83)
-   expect_equivalent(round(res.FE$zval,2), 1.66) ### 1.65 in chapter
+   expect_equivalent(coef(res.FE), 0.0604, tolerance=.tol[["coef"]])
+   expect_equivalent(res.FE$QE,   35.8295, tolerance=.tol[["test"]])
+   expect_equivalent(res.FE$zval,  1.6553, tolerance=.tol[["test"]]) ### 1.65 in chapter
 
 })
 
@@ -25,22 +27,22 @@ test_that("results are correct for the random-effects model.", {
    res.RE <- rma(yi, vi, data=dat, digits=3)
 
    ### compare with results on page 301 (Table 16.2) and page 302
-   expect_equivalent(round(coef(res.RE),3), 0.084) ### 0.083 in chapter
-   expect_equivalent(round(res.RE$zval,2), 1.62)
-   expect_equivalent(round(res.RE$tau2,3), 0.019)
+   expect_equivalent(coef(res.RE), 0.0837, tolerance=.tol[["coef"]]) ### 0.083 in chapter
+   expect_equivalent(res.RE$zval,  1.6208, tolerance=.tol[["test"]])
+   expect_equivalent(res.RE$tau2,  0.0188, tolerance=.tol[["var"]])
 
-   ### credibility/prediction interval
+   ### prediction interval
    tmp <- predict(res.RE)
 
    ### compare with results on page 301 (Table 16.2) and page 302
-   expect_equivalent(round(tmp$cr.lb,3), -0.204) ### -0.19 in chapter but computed in a slightly different way
-   expect_equivalent(round(tmp$cr.ub,3),  0.371) ###  0.35 in chapter but computed in a slightly different way
+   expect_equivalent(tmp$pi.lb, -0.2036, tolerance=.tol[["ci"]]) ### -0.19 in chapter but computed in a slightly different way
+   expect_equivalent(tmp$pi.ub,  0.3711, tolerance=.tol[["ci"]]) ###  0.35 in chapter but computed in a slightly different way
 
    ### range of BLUPs
-   tmp <- round(range(blup(res.RE)$pred), 2)
+   tmp <- range(blup(res.RE)$pred)
 
    ### compare with results on page 301 (Table 16.2)
-   expect_equivalent(tmp, c(-0.03, 0.25))
+   expect_equivalent(tmp, c(-0.0293, 0.2485), tolerance=.tol[["pred"]])
 
 })
 
@@ -53,16 +55,16 @@ test_that("results are correct for the mixed-effects model.", {
    res.ME <- rma(yi, vi, mods = ~ weeks.c, data=dat, digits=3)
 
    ### compare with results on page 301 (Table 16.2)
-   expect_equivalent(round(res.ME$tau2,3), 0)
-   expect_equivalent(round(coef(res.ME),3), c(0.407, -0.157))
-   expect_equivalent(round(res.ME$QE,2), 16.57)
-   expect_equivalent(round(res.ME$zval,2), c(4.68, -4.39))
+   expect_equivalent(res.ME$tau2, 0.0000, tolerance=.tol[["var"]])
+   expect_equivalent(coef(res.ME), c(0.4072, -0.1572), tolerance=.tol[["coef"]])
+   expect_equivalent(res.ME$QE, 16.5708, tolerance=.tol[["test"]])
+   expect_equivalent(res.ME$zval, c(4.6782, -4.3884), tolerance=.tol[["test"]])
 
    ### range of BLUPs
-   tmp <- round(range(blup(res.ME)$pred), 2)
+   tmp <- range(blup(res.ME)$pred)
 
    ### compare with results on page 301 (Table 16.2)
-   expect_equivalent(tmp, c(-0.06, 0.41)) ### -0.07 in chapter
+   expect_equivalent(tmp, c(-0.0646, 0.4072), tolerance=.tol[["pred"]]) ### -0.07 in chapter
 
 })
 
@@ -76,13 +78,13 @@ test_that("results are correct for the random-effects model (conventional approa
    res.std$DL   <- rma(yi, vi, data=dat, digits=3, method="DL")
    res.std$HE   <- rma(yi, vi, data=dat, digits=3, method="HE")
 
-   tmp <- round(t(sapply(res.std, function(x) c(tau2=x$tau2, mu=x$beta, se=x$se, z=x$zval, ci.lb=x$ci.lb, ci.ub=x$ci.ub))), 3)
+   tmp <- t(sapply(res.std, function(x) c(tau2=x$tau2, mu=x$beta, se=x$se, z=x$zval, ci.lb=x$ci.lb, ci.ub=x$ci.ub)))
 
-   expected <- structure(c(0, 0.013, 0.019, 0.026, 0.08, 0.06, 0.078, 0.084, 0.089, 0.114, 0.036, 0.047, 0.052, 0.056, 0.079, 1.655, 1.637, 1.621, 1.601, 1.443, -0.011, -0.015, -0.018, -0.02, -0.041, 0.132, 0.171, 0.185, 0.199, 0.27),
+   expected <- structure(c(0, 0.0126, 0.0188, 0.0259, 0.0804, 0.0604, 0.0777, 0.0837, 0.0893, 0.1143, 0.0365, 0.0475, 0.0516, 0.0558, 0.0792, 1.6553, 1.6368, 1.6208, 1.6009, 1.4432, -0.0111, -0.0153, -0.0175, -0.02, -0.0409, 0.1318, 0.1708, 0.1849, 0.1987, 0.2696),
                          .Dim = 5:6, .Dimnames = list(c("FE", "ML", "REML", "DL", "HE"), c("tau2", "mu", "se", "z", "ci.lb", "ci.ub")))
 
    ### compare with results on page 309 (Table 16.3)
-   expect_equivalent(tmp, expected)
+   expect_equivalent(tmp, expected, tolerance=.tol[["misc"]])
 
 })
 
@@ -96,13 +98,13 @@ test_that("results are correct for the random-effects model (Knapp & Hartung met
    res.knha$DL   <- rma(yi, vi, data=dat, digits=3, method="DL", test="knha")
    res.knha$HE   <- rma(yi, vi, data=dat, digits=3, method="HE", test="knha")
 
-   tmp <- round(t(sapply(res.knha, function(x) c(tau2=x$tau2, mu=x$beta, se=x$se, z=x$zval, ci.lb=x$ci.lb, ci.ub=x$ci.ub))), 3)
+   tmp <- t(sapply(res.knha, function(x) c(tau2=x$tau2, mu=x$beta, se=x$se, z=x$zval, ci.lb=x$ci.lb, ci.ub=x$ci.ub)))
 
-   expected <- structure(c(0, 0.013, 0.019, 0.026, 0.08, 0.06, 0.078, 0.084, 0.089, 0.114, 0.051, 0.059, 0.062, 0.064, 0.071, 1.173, 1.311, 1.359, 1.405, 1.608, -0.048, -0.047, -0.046, -0.044, -0.035, 0.168, 0.202, 0.213, 0.223, 0.264),
+   expected <- structure(c(0, 0.0126, 0.0188, 0.0259, 0.0804, 0.0604, 0.0777, 0.0837, 0.0893, 0.1143, 0.0515, 0.0593, 0.0616, 0.0636, 0.0711, 1.1733, 1.311, 1.3593, 1.405, 1.6078, -0.0477, -0.0468, -0.0457, -0.0442, -0.0351, 0.1685, 0.2023, 0.2131, 0.2229, 0.2637),
                          .Dim = 5:6, .Dimnames = list(c("FE", "ML", "REML", "DL", "HE"), c("tau2", "mu", "se", "z", "ci.lb", "ci.ub")))
 
    ### compare with results on page 309 (Table 16.3)
-   expect_equivalent(tmp, expected)
+   expect_equivalent(tmp, expected, tolerance=.tol[["misc"]])
 
 })
 
@@ -126,12 +128,12 @@ test_that("results are correct for the random-effects model (Huber-White method)
 
    out <- capture.output(print(res.hw$REML)) ### so that print.robust.rma() is run (at least once)
 
-   tmp <- round(t(sapply(res.hw, function(x) c(tau2=x$tau2, mu=x$beta, se=x$se, t=x$zval, ci.lb=x$ci.lb, ci.ub=x$ci.ub))), 3)
+   tmp <- t(sapply(res.hw, function(x) c(tau2=x$tau2, mu=x$beta, se=x$se, t=x$zval, ci.lb=x$ci.lb, ci.ub=x$ci.ub)))
 
-   expected <- structure(c(0, 0.013, 0.019, 0.026, 0.08, 0.06, 0.078, 0.084, 0.089, 0.114, 0.04, 0.047, 0.05, 0.052, 0.062, 1.515, 1.637, 1.676, 1.71, 1.85, -0.023, -0.022, -0.021, -0.02, -0.015, 0.144, 0.178, 0.189, 0.199, 0.244),
+   expected <- structure(c(0, 0.0126, 0.0188, 0.0259, 0.0804, 0.0604, 0.0777, 0.0837, 0.0893, 0.1143, 0.0398, 0.0475, 0.05, 0.0522, 0.0618, 1.5148, 1.6369, 1.6756, 1.7105, 1.8503, -0.0234, -0.022, -0.0213, -0.0204, -0.0155, 0.1441, 0.1775, 0.1887, 0.199, 0.2441),
                          .Dim = 5:6, .Dimnames = list(c("FE", "ML", "REML", "DL", "HE"), c("tau2", "mu", "se", "t", "ci.lb", "ci.ub")))
 
    ### compare with results on page 309 (Table 16.3)
-   expect_equivalent(tmp, expected)
+   expect_equivalent(tmp, expected, tolerance=.tol[["misc"]])
 
 })
